@@ -28,7 +28,6 @@ from mypy_extensions import mypyc_attr
 from typing_extensions import Protocol, get_args, get_origin
 
 if TYPE_CHECKING:
-
     from typing_extensions import Literal, TypeGuard
 
     ReducerFunc = Callable[[tuple, tuple], tuple]
@@ -110,7 +109,6 @@ class Signal:
         check_nargs_on_connect: bool = True,
         check_types_on_connect: bool = False,
     ) -> None:
-
         self._name = name
         self.description = description
         self._check_nargs_on_connect = check_nargs_on_connect
@@ -292,7 +290,15 @@ class SignalInstance:
         check_types_on_connect: bool = False,
     ) -> None:
         self._name = name
-        self._instance: Any = instance
+        if instance is None:
+            self._instance: Callable = lambda: None
+        else:
+            try:
+                self._instance = weakref.ref(instance)
+            except TypeError:
+                # fall back to strong reference if instance is not weak-referenceable
+                self._instance = lambda: instance
+
         self._args_queue: list[Any] = []  # filled when paused
 
         if isinstance(signature, (list, tuple)):
@@ -319,7 +325,7 @@ class SignalInstance:
     @property
     def instance(self) -> Any:
         """Object that emits this `SignalInstance`."""
-        return self._instance
+        return self._instance()
 
     @property
     def name(self) -> str:
