@@ -16,7 +16,6 @@ class MyStrictGroup(SignalGroup, strict=True):
 
 
 def test_signal_group():
-
     assert not MyGroup.is_uniform()
     group = MyGroup()
     assert not group.is_uniform()
@@ -46,7 +45,6 @@ def test_uniform_group():
 
 @pytest.mark.parametrize("direct", [True, False])
 def test_signal_group_connect(direct: bool):
-
     mock = Mock()
     group = MyGroup()
     if direct:
@@ -168,3 +166,18 @@ def test_group_disconnect_all_slots():
 
     mock1.assert_not_called()
     mock2.assert_not_called()
+
+
+def test_weakref():
+    """Make sure that the group doesn't keep a strong reference to the instance."""
+    import gc
+
+    class T:
+        ...
+
+    obj = T()
+    group = MyGroup(obj)
+    assert group.instance is obj
+    del obj
+    gc.collect()
+    assert group.instance is None
